<?php

namespace Bixo\Employee\Http\Controllers;

use Exception;
use Bixo\Employee\Actions\AttendanceWorkflow;
use Bixo\Employee\Http\Requests\AttendanceWorkflowRequest;
use Bixo\Employee\Http\Resources\AttendanceResource;
use Bixo\Employee\Models\Attendance;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class AttendanceWorkflowController extends BaseController
{
    /**
     * Action controller function for attendance.
     *
     * @param Model $attendance
     * @param action next action for the attendance.
     *
     * @return Response
     */
    public function __invoke(AttendanceWorkflowRequest $request, Attendance $attendance, $trasition)
    {
        try {
            $request = $request->all();
            $attendance = AttendanceWorkflow::run($trasition, $attendance, $request);
            $data = new AttendanceResource($attendance);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('employee::attendance.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('employee/attendance/' . $attendance->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('employee/attendance/' . $attendance->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the attendance and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $attendance
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $attendance, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $attendance = Attendance::findBySignedId($attendance);
        $transitions = $this->transitions($attendance, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('employee::attendance.name') . ' ' . trans('Approvals'))
            ->view('employee::public.attendance.workflow')
            ->layout('mobile')
            ->data(compact('attendance', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the attendance.
     *
     * @param PublicRequest $request
     * @param string $attendance
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $attendance, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $attendance = Attendance::findBySignedId($attendance);
        $user = user();
        $attendance = AttendanceWorkflow::run($transition, $attendance, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($attendance, $user)
    {
        $transitions = [];
        foreach ($attendance->workflow()->transitions($attendance) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.attendance.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'attendance' => $attendance->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $attendance->workflow()->form($value);
            $array['label'] = trans('employee::attendance.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
